<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

// @codingStandardsIgnoreFile

namespace Module\Agilecrm\Block;
use Magento\Store\Model\Store;

/**
 * GoogleAnalytics Page Block
 */
class Ga extends \Magento\Framework\View\Element\Template
{

    const XML_PATH_SITE = 'agilesettings/analytics/domainname';
    const XML_PATH_API_KEY = 'agilesettings/analytics/jsapikey';
    /**
     * Google analytics data
     *
     * @var \Magento\GoogleAnalytics\Helper\Data
     */
    protected $_googleAnalyticsData = null;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $_salesOrderCollection;

    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $salesOrderCollection
     * @param \Magento\GoogleAnalytics\Helper\Data $googleAnalyticsData
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $salesOrderCollection,
        \Module\Agilecrm\Helper\Data $googleAnalyticsData,
        array $data = []
    ) {
        $this->_googleAnalyticsData = $googleAnalyticsData;
        $this->_salesOrderCollection = $salesOrderCollection;
        parent::__construct($context, $data);
    }

    /**
     * Get config
     *
     * @param string $path
     * @return mixed
     */
    public function getConfig($path)
    {
        return $this->_scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    /**
     * Get a specific page name (may be customized via layout)
     *
     * @return string|null
     */
    public function getPageName()
    {
        return $this->_getData('page_name');
    }

    /**
     * Render regular page tracking javascript code
     * The custom "page name" may be set from layout or somewhere else. It must start from slash.
     *
     * @param string $accountId
     * @return string
     * @link https://developers.google.com/analytics/devguides/collection/analyticsjs/method-reference#set
     * @link https://developers.google.com/analytics/devguides/collection/analyticsjs/method-reference#gaObjectMethods
     */
    public function getPageTrackingCode($accountId)
    {
        $pageName = trim($this->getPageName());
        $optPageURL = '';
        if ($pageName && substr($pageName, 0, 1) == '/' && strlen($pageName) > 1) {
            $optPageURL = ", '{$this->escapeJsQuote($pageName)}'";
        }
        $om = \Magento\Framework\App\ObjectManager::getInstance();
           $syncustomer = $this->_googleAnalyticsData->isagilecrmsyncustomer();
              if ($syncustomer == 1) {
                $customerSession = $om->get('Magento\Customer\Model\Session');
                $customername = $customerSession->getCustomer()->getName();  // get  Full Name
                $customername = explode(" ",$customername);
                $customeremail = $customerSession->getCustomer()->getEmail(); // get Email Name
                $contact_json = array(
                  "tags"=>array("Magento"),
                  "properties"=>array(
                    array(
                      "name"=>"first_name",
                      "value"=>$customername[0],
                      "type"=>"SYSTEM"
                    ),
                     array(
                      "name"=>"last_name",
                      "value"=>$customername[1],
                      "type"=>"SYSTEM"
                    ),
                    array(
                      "name"=>"email",
                      "value"=>$customeremail,
                      "type"=>"SYSTEM"
                    ),  
                  )
                );
                $contact_json = json_encode($contact_json);
                $curln = $this->curl_wrap("contacts", $contact_json, "POST", "application/json");
               }
        return "";
    }

    /**
     * Render information about specified orders and their items
     *
     * @link https://developers.google.com/analytics/devguides/collection/analyticsjs/enhanced-ecommerce#checkout-options
     * @link https://developers.google.com/analytics/devguides/collection/analyticsjs/enhanced-ecommerce#measuring-transactions
     * @link https://developers.google.com/analytics/devguides/collection/analyticsjs/enhanced-ecommerce#transaction
     *
     * @return string|void
     */
    public function getOrdersTrackingCode()
    {
        $orderIds = $this->getOrderIds();
        if (empty($orderIds) || !is_array($orderIds)) {
            return;
        }

        $collection = $this->_salesOrderCollection->create();
        $collection->addFieldToFilter('entity_id', ['in' => $orderIds]);
        foreach ($collection as $order) {
            $entity = $order->getData();
            if ($order->getIsVirtual()) {
               $address = $order->getBillingAddress();
               $billAddr = $address->getData();
            } else {
               $address = $order->getShippingAddress();
                $billAddr = $address->getData();
            } 
            $syncustomer = $this->_googleAnalyticsData->isagilecrmsyncustomer();
            $syncorder = $this->_googleAnalyticsData->isagilecrmsyncorder();
              if ($syncustomer == 1) {
                
                $customerfirst_name = $entity['customer_firstname'];
                $customerlast_name = $entity['customer_lastname'];
                if($customerfirst_name == null){
                $customername = $order->getBillingAddress()->getName();
                $customername = explode(" ",$customername);
                $customerfirst_name = $customername[0];
                $customerlast_name = $customername[1];
                }
                $customeremail = $entity['customer_email'];
                $customerphone = $billAddr['telephone'];
                $customercompany = $billAddr['company'];
                if($customercompany == null){
                $customercompany= $order->getBillingAddress()->getCompany();
                }
                $countryid = $this->getCountryName($billAddr['country_id']);
                $zipcode = $address->getPostcode();
                $street = $billAddr['street'];
                $city = $billAddr['city'];
                $region = $billAddr['region'];
                $zip = $zipcode;
                $address = array(
                  "address"=>$street,
                  "city"=>$city,
                  "state"=>$region,
                  "zip"=>$zip,
                  "country"=>$countryid
                );
                $contact_json = array(
                  "tags"=>array("Magento"),
                  "properties"=>array(
                    array(
                      "name"=>"first_name",
                      "value"=>$customerfirst_name,
                      "type"=>"SYSTEM"
                    ),
                    array(
                      "name"=>"last_name",
                      "value"=>$customerlast_name,
                      "type"=>"SYSTEM"
                    ),
                    array(
                      "name"=>"email",
                      "value"=>$customeremail,
                      "type"=>"SYSTEM"
                    ), 
                    array(
                      "name"=>"company",
                      "value"=>$customercompany,
                      "type"=>"SYSTEM"
                    ),   
                    array(
                        "name"=>"address",
                        "value"=>json_encode($address),
                        "type"=>"SYSTEM"
                    ),
                    array(
                        "name"=>"phone",
                        "value"=>$customerphone,
                        "type"=>"SYSTEM"
                    )
                  )
                );
                $contact_json = json_encode($contact_json);
                $curln = $this->curl_wrap("contacts", $contact_json, "POST", "application/json");
               }

               if ($syncorder == 1) {
                  $result = $this->curl_wrap("contacts/search/email/".$customeremail, null, "GET", "application/json");
                    $result = json_decode($result, false, 512, JSON_BIGINT_AS_STRING);
                    $contact_id = $result->id;
                    $productname = array();
                    foreach ($order->getAllVisibleItems() as $item) {
                    $productname[] =$this->escapeJsQuote($item->getName());
                    }
                    $noteproductname = implode(',',$productname);
                    $productname = implode('","',$productname);
                    $Str = $productname;
                    $Str = preg_replace('/[^a-zA-Z0-9_ ,".]/', '_', $Str);
                    $contact_json = array(
                        "id" => $contact_id, 
                       "tags" => array($Str,"Magento")
                    );
                   $contact_json = stripslashes(json_encode($contact_json));
                   $curltags = $this->curl_wrap("contacts/edit/tags", $contact_json, "PUT", "application/json");
                   $billingaddress = $street.",".$city.",".$region.",".$countryid;
                   $grandtotal = $order->getBaseGrandTotal();
                   $orderid = $order->getIncrementId();
                   $description = "Order status: On Hold\nTotal amount:".$grandtotal."\nItems(id-qty):".$noteproductname."\nBilling:".$billingaddress;
                   $note_json = array(
                      "subject"=>$orderid,
                      "description"=>$description,
                      "contact_ids"=>array($contact_id)
                    );
                $note_json = json_encode($note_json);
                $curls = $this->curl_wrap("notes", $note_json, "POST", "application/json");
                $address = array(
                  "address"=>$street,
                  "city"=>$city,
                  "state"=>$region,
                  "zip" =>$zip,
                  "country"=>$countryid
                );
               $contact_json_update = array(
              "id"=>$contact_id, //It is mandatory field. Id of contact
              "properties"=>array(
               array(
                  "name"=>"first_name",
                  "value"=>$customerfirst_name,
                  "type"=>"SYSTEM"
                ),
                array(
                  "name"=>"last_name",
                  "value"=>$customerlast_name,
                  "type"=>"SYSTEM"
                ), 
                array(
                  "name"=>"company",
                  "value"=>$customercompany,
                  "type"=>"SYSTEM"
                ),   
                array(
                    "name"=>"phone",
                    "value"=>$customerphone,
                    "type"=>"SYSTEM"
                ),
               array(
                        "name"=>"address",
                        "value"=>json_encode($address),
                        "type"=>"SYSTEM"
               )
              )
            );

           $contact_json_update = json_encode($contact_json_update);
           $curlupdate = $this->curl_wrap("contacts/edit-properties", $contact_json_update, "PUT", "application/json");
               
           }
       }
        return ;
    }

    /**
     * Render GA tracking scripts
     *
     * @return string
     */
    protected function _toHtml()
    {
        if (!$this->_googleAnalyticsData->isagilecrmAvailable()) {
            return '';
        }
        return parent::_toHtml();
    }
   public function get_contact_id(){
        $result = $this->curl_wrap("api-key", null, "GET", "application/json");
        $result = json_decode($result, True, 512, JSON_BIGINT_AS_STRING);
        $id = $result['id'];
      return $id;
   } 
    public function curl_wrap($entity, $data, $method, $content_type) {
      $doaminname =$this->_googleAnalyticsData->isagilecrmpostdomain();
      $oweremail = $this->_googleAnalyticsData->isagilecrmoweremailid();
      $restapikey = $this->_googleAnalyticsData->isagilecrmrestapi();
    if ($content_type == NULL) {
        $content_type = "application/json";
    }
     $agile_url = "https://" . $doaminname . ".agilecrm.com/dev/api/" . $entity;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_UNRESTRICTED_AUTH, true);
        switch ($method) {
            case "POST":
                $url = $agile_url;
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;
            case "GET":
                $url = $agile_url;
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");
                break;
            case "PUT":
                $url = $agile_url;
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;
            case "DELETE":
                $url = $agile_url;
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
                break;
            default:
                break;
        }
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Content-type : $content_type;", 'Accept : application/json'
        ));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_USERPWD, $oweremail . ':' . $restapikey);
        curl_setopt($ch, CURLOPT_TIMEOUT, 120);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $output = curl_exec($ch);
        curl_close($ch);
        return $output;
     }

    private function getCountryName($ccode)
    {
        $countries = array
            (
            'AF' => 'Afghanistan',
            'AX' => 'Aland Islands',
            'AL' => 'Albania',
            'DZ' => 'Algeria',
            'AS' => 'American Samoa',
            'AD' => 'Andorra',
            'AO' => 'Angola',
            'AI' => 'Anguilla',
            'AQ' => 'Antarctica',
            'AG' => 'Antigua And Barbuda',
            'AR' => 'Argentina',
            'AM' => 'Armenia',
            'AW' => 'Aruba',
            'AU' => 'Australia',
            'AT' => 'Austria',
            'AZ' => 'Azerbaijan',
            'BS' => 'Bahamas',
            'BH' => 'Bahrain',
            'BD' => 'Bangladesh',
            'BB' => 'Barbados',
            'BY' => 'Belarus',
            'BE' => 'Belgium',
            'BZ' => 'Belize',
            'BJ' => 'Benin',
            'BM' => 'Bermuda',
            'BT' => 'Bhutan',
            'BO' => 'Bolivia',
            'BA' => 'Bosnia And Herzegovina',
            'BW' => 'Botswana',
            'BV' => 'Bouvet Island',
            'BR' => 'Brazil',
            'IO' => 'British Indian Ocean Territory',
            'BN' => 'Brunei Darussalam',
            'BG' => 'Bulgaria',
            'BF' => 'Burkina Faso',
            'BI' => 'Burundi',
            'KH' => 'Cambodia',
            'CM' => 'Cameroon',
            'CA' => 'Canada',
            'CV' => 'Cape Verde',
            'KY' => 'Cayman Islands',
            'CF' => 'Central African Republic',
            'TD' => 'Chad',
            'CL' => 'Chile',
            'CN' => 'China',
            'CX' => 'Christmas Island',
            'CC' => 'Cocos (Keeling) Islands',
            'CO' => 'Colombia',
            'KM' => 'Comoros',
            'CG' => 'Congo',
            'CD' => 'Congo, Democratic Republic',
            'CK' => 'Cook Islands',
            'CR' => 'Costa Rica',
            'CI' => 'Cote D\'Ivoire',
            'HR' => 'Croatia',
            'CU' => 'Cuba',
            'CY' => 'Cyprus',
            'CZ' => 'Czech Republic',
            'DK' => 'Denmark',
            'DJ' => 'Djibouti',
            'DM' => 'Dominica',
            'DO' => 'Dominican Republic',
            'EC' => 'Ecuador',
            'EG' => 'Egypt',
            'SV' => 'El Salvador',
            'GQ' => 'Equatorial Guinea',
            'ER' => 'Eritrea',
            'EE' => 'Estonia',
            'ET' => 'Ethiopia',
            'FK' => 'Falkland Islands (Malvinas)',
            'FO' => 'Faroe Islands',
            'FJ' => 'Fiji',
            'FI' => 'Finland',
            'FR' => 'France',
            'GF' => 'French Guiana',
            'PF' => 'French Polynesia',
            'TF' => 'French Southern Territories',
            'GA' => 'Gabon',
            'GM' => 'Gambia',
            'GE' => 'Georgia',
            'DE' => 'Germany',
            'GH' => 'Ghana',
            'GI' => 'Gibraltar',
            'GR' => 'Greece',
            'GL' => 'Greenland',
            'GD' => 'Grenada',
            'GP' => 'Guadeloupe',
            'GU' => 'Guam',
            'GT' => 'Guatemala',
            'GG' => 'Guernsey',
            'GN' => 'Guinea',
            'GW' => 'Guinea-Bissau',
            'GY' => 'Guyana',
            'HT' => 'Haiti',
            'HM' => 'Heard Island & Mcdonald Islands',
            'VA' => 'Holy See (Vatican City State)',
            'HN' => 'Honduras',
            'HK' => 'Hong Kong',
            'HU' => 'Hungary',
            'IS' => 'Iceland',
            'IN' => 'India',
            'ID' => 'Indonesia',
            'IR' => 'Iran, Islamic Republic Of',
            'IQ' => 'Iraq',
            'IE' => 'Ireland',
            'IM' => 'Isle Of Man',
            'IL' => 'Israel',
            'IT' => 'Italy',
            'JM' => 'Jamaica',
            'JP' => 'Japan',
            'JE' => 'Jersey',
            'JO' => 'Jordan',
            'KZ' => 'Kazakhstan',
            'KE' => 'Kenya',
            'KI' => 'Kiribati',
            'KR' => 'Korea',
            'KW' => 'Kuwait',
            'KG' => 'Kyrgyzstan',
            'LA' => 'Lao People\'s Democratic Republic',
            'LV' => 'Latvia',
            'LB' => 'Lebanon',
            'LS' => 'Lesotho',
            'LR' => 'Liberia',
            'LY' => 'Libyan Arab Jamahiriya',
            'LI' => 'Liechtenstein',
            'LT' => 'Lithuania',
            'LU' => 'Luxembourg',
            'MO' => 'Macao',
            'MK' => 'Macedonia',
            'MG' => 'Madagascar',
            'MW' => 'Malawi',
            'MY' => 'Malaysia',
            'MV' => 'Maldives',
            'ML' => 'Mali',
            'MT' => 'Malta',
            'MH' => 'Marshall Islands',
            'MQ' => 'Martinique',
            'MR' => 'Mauritania',
            'MU' => 'Mauritius',
            'YT' => 'Mayotte',
            'MX' => 'Mexico',
            'FM' => 'Micronesia, Federated States Of',
            'MD' => 'Moldova',
            'MC' => 'Monaco',
            'MN' => 'Mongolia',
            'ME' => 'Montenegro',
            'MS' => 'Montserrat',
            'MA' => 'Morocco',
            'MZ' => 'Mozambique',
            'MM' => 'Myanmar',
            'NA' => 'Namibia',
            'NR' => 'Nauru',
            'NP' => 'Nepal',
            'NL' => 'Netherlands',
            'AN' => 'Netherlands Antilles',
            'NC' => 'New Caledonia',
            'NZ' => 'New Zealand',
            'NI' => 'Nicaragua',
            'NE' => 'Niger',
            'NG' => 'Nigeria',
            'NU' => 'Niue',
            'NF' => 'Norfolk Island',
            'MP' => 'Northern Mariana Islands',
            'NO' => 'Norway',
            'OM' => 'Oman',
            'PK' => 'Pakistan',
            'PW' => 'Palau',
            'PS' => 'Palestinian Territory, Occupied',
            'PA' => 'Panama',
            'PG' => 'Papua New Guinea',
            'PY' => 'Paraguay',
            'PE' => 'Peru',
            'PH' => 'Philippines',
            'PN' => 'Pitcairn',
            'PL' => 'Poland',
            'PT' => 'Portugal',
            'PR' => 'Puerto Rico',
            'QA' => 'Qatar',
            'RE' => 'Reunion',
            'RO' => 'Romania',
            'RU' => 'Russian Federation',
            'RW' => 'Rwanda',
            'BL' => 'Saint Barthelemy',
            'SH' => 'Saint Helena',
            'KN' => 'Saint Kitts And Nevis',
            'LC' => 'Saint Lucia',
            'MF' => 'Saint Martin',
            'PM' => 'Saint Pierre And Miquelon',
            'VC' => 'Saint Vincent And Grenadines',
            'WS' => 'Samoa',
            'SM' => 'San Marino',
            'ST' => 'Sao Tome And Principe',
            'SA' => 'Saudi Arabia',
            'SN' => 'Senegal',
            'RS' => 'Serbia',
            'SC' => 'Seychelles',
            'SL' => 'Sierra Leone',
            'SG' => 'Singapore',
            'SK' => 'Slovakia',
            'SI' => 'Slovenia',
            'SB' => 'Solomon Islands',
            'SO' => 'Somalia',
            'ZA' => 'South Africa',
            'GS' => 'South Georgia And Sandwich Isl.',
            'ES' => 'Spain',
            'LK' => 'Sri Lanka',
            'SD' => 'Sudan',
            'SR' => 'Suriname',
            'SJ' => 'Svalbard And Jan Mayen',
            'SZ' => 'Swaziland',
            'SE' => 'Sweden',
            'CH' => 'Switzerland',
            'SY' => 'Syrian Arab Republic',
            'TW' => 'Taiwan',
            'TJ' => 'Tajikistan',
            'TZ' => 'Tanzania',
            'TH' => 'Thailand',
            'TL' => 'Timor-Leste',
            'TG' => 'Togo',
            'TK' => 'Tokelau',
            'TO' => 'Tonga',
            'TT' => 'Trinidad And Tobago',
            'TN' => 'Tunisia',
            'TR' => 'Turkey',
            'TM' => 'Turkmenistan',
            'TC' => 'Turks And Caicos Islands',
            'TV' => 'Tuvalu',
            'UG' => 'Uganda',
            'UA' => 'Ukraine',
            'AE' => 'United Arab Emirates',
            'GB' => 'United Kingdom',
            'US' => 'United States',
            'UM' => 'United States Outlying Islands',
            'UY' => 'Uruguay',
            'UZ' => 'Uzbekistan',
            'VU' => 'Vanuatu',
            'VE' => 'Venezuela',
            'VN' => 'Viet Nam',
            'VG' => 'Virgin Islands, British',
            'VI' => 'Virgin Islands, U.S.',
            'WF' => 'Wallis And Futuna',
            'EH' => 'Western Sahara',
            'YE' => 'Yemen',
            'ZM' => 'Zambia',
            'ZW' => 'Zimbabwe',
        );
        return isset($countries[$ccode]) ? $countries[$ccode] : $ccode;
    }
}
